package de.fraunhofer.iem.dataprovider.repository.controller

import de.fraunhofer.iem.dataprovider.configuration.ApiPaths
import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeResponseDto
import de.fraunhofer.iem.dataprovider.kpi.service.KPIService
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RawKpiDto
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryResponseDto
import de.fraunhofer.iem.dataprovider.repository.dto.ScoreCardResponseDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.toolRun.dto.ToolRunDto
import de.fraunhofer.iem.dataprovider.toolRun.service.ToolRunService
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RestController
import java.util.*

@RestController
class RepositoryController(
    private val repositoryService: RepositoryService,
    private val kpiService: KPIService,
    private val toolRunService: ToolRunService
) {

    private val logger = getLogger(javaClass)

    @GetMapping(ApiPaths.REPOSITORY)
    suspend fun getAllRepositories(): List<RepositoryResponseDto> {
        logger.info("Get all repositories")
        return repositoryService.getAllRepositories().map { repositoryEntity: RepositoryEntity ->
            RepositoryResponseDto(
                id = repositoryEntity.id!!,
                name = repositoryEntity.name,
                repositoryId = repositoryEntity.repoId,
                url = repositoryEntity.url
            )
        }
    }

    @GetMapping(ApiPaths.REPOSITORY_SCORE_CARD)
    suspend fun getAllScoreCards(): List<ScoreCardResponseDto> {
        logger.info("Get all repositories as score cards")
        return repositoryService.getAllRepositories().map { repositoryEntity: RepositoryEntity ->
            val rootKpi = kpiService.removeKPIChildrenLowerThanSecondLevel(
                this.kpiService.getKpiTreeForRepository(repositoryEntity).toViewModel()
            )

            ScoreCardResponseDto(
                RepositoryResponseDto(
                    id = repositoryEntity.id!!,
                    name = repositoryEntity.name,
                    repositoryId = repositoryEntity.repoId,
                    url = repositoryEntity.url
                ),
                rootKpi
            )
        }
    }

    @GetMapping(ApiPaths.SCORE_CARD_BY_REPOSITORY_ID)
    suspend fun getScoreCardByRepositoryId(@PathVariable id: UUID): ScoreCardResponseDto {
        logger.info("Get repository score card with id $id")
        val repositoryEntity = this.repositoryService.findRepoByID(id) ?: throw Exception("Repository can not be found")
        val rootKpi = kpiService.removeKPIChildrenLowerThanSecondLevel(
            this.kpiService.getKpiTreeForRepository(repositoryEntity).toViewModel()
        )

        return ScoreCardResponseDto(
            RepositoryResponseDto(
                id = repositoryEntity.id!!,
                name = repositoryEntity.name,
                repositoryId = repositoryEntity.repoId,
                url = repositoryEntity.url
            ),
            rootKpi
        )
    }

    @GetMapping(ApiPaths.REPOSITORY_ID)
    suspend fun getRepositoryById(@PathVariable id: Long): RepositoryResponseDto {
        logger.info("Get repository with id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw Exception("Repository can not be found")
        return RepositoryResponseDto(
            id = repositoryEntity.id!!,
            name = repositoryEntity.name,
            repositoryId = repositoryEntity.repoId,
            url = repositoryEntity.url
        )
    }

    /**
     * This endpoint is mainly used for debug calls to verify the db contents.
     */
    @GetMapping(ApiPaths.RAW_KPI_BY_REPOSITORY_ID)
    suspend fun getRawKPIByRepositoryId(@PathVariable id: Long): RawKpiDto {
        logger.info("Get raw KPIs for repository id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw Exception("Repository can not be found")
        val rawKpis = this.kpiService.getRawKpisForRepository(repositoryEntity)
        return RawKpiDto(rawKpis = rawKpis)
    }

    @GetMapping(ApiPaths.KPI_BY_REPOSITORY_ID)
    suspend fun getKPIByRepositoryId(@PathVariable id: Long): KPITreeResponseDto {
        logger.info("Get KPI tree for repository id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw Exception("Repository can not be found")
        val rootKpi = this.kpiService.getKpiTreeForRepository(repositoryEntity)
        return rootKpi.toViewModel()
    }

    @GetMapping(ApiPaths.REPOSITORY_TOOLS)
    suspend fun getToolsForRepository(@PathVariable id: Long): ToolRunDto {
        logger.info("Get repository with id $id")
        val repositoryEntity = this.repositoryService.findRepoById(id) ?: throw Exception("Repository can not be found")

        return toolRunService.getToolRunForRepository(repositoryEntity)
    }
}
